<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\Esb\Model\Rest;
use FiloBlu\Esb\Api\ExternalShipmentInterface;
use Magento\Sales\Api\Data\ShipmentInterface;

/**
 * REST Api to save external shipments
 */
class ExternalShipment implements ExternalShipmentInterface{
	
    /**
     * @var \Magento\Sales\Model\OrderRepository
     */ 	
    protected $_orderRepository;

    /**
     * @var \Magento\Sales\Model\Order\ShipmentFactory 
     */ 	
    protected $_shipmentFactory;
    
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder 
     */ 
    protected $_searchCriteriaBuilder;

    /**
     * @var \Magento\Sales\Model\Order\ShipmentFactory 
     */ 	
    protected $_shipmentNotifier;
    
    
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $_orderExtensionFactory;

    /**
     * @var \Monolog\Logger
     */ 
    protected $_logger;
    
    /**
     * @var \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */     
    protected $_response;
    
    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Sales\Model\Order\ShipmentFactory $shipmentFactory     
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder     
     */
    public function __construct(
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \Magento\Sales\Model\OrderRepository $orderRepository,
            \Magento\Sales\Model\Order\ShipmentFactory $shipmentFactory,
            \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
            \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier,
            \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
            \FiloBlu\Esb\Helper\LoggerProvider $loggerProvider,
            \FiloBlu\Esb\Api\Data\AcknowledgeInterface $response            
    ){
        $this->_scopeConfig = $scopeConfig;
        $this->_orderRepository = $orderRepository;
        $this->_shipmentFactory = $shipmentFactory;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_shipmentNotifier = $shipmentNotifier;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;         
        $this->_logger=$loggerProvider->getLogger();
        $this->_response = $response;
    }

    /**
     * Save External Shipment data. Save PDF file and additional data on Sales Order.
     *
     * @param \Magento\Sales\Api\Data\ShipmentInterface  $shipment_data
     * @return int
     */
     public function save(ShipmentInterface $shipment_data){
        $shipmentExtensionAttributes=$shipment_data->getExtensionAttributes();
        if(null==$shipmentExtensionAttributes || !$shipmentExtensionAttributes->getOrderIncrementId()){
            $message=__("Cannot find order to ship");
            $this->_logger->critical($message);
            throw new Magento\Sales\Exception\CouldNotShipException($message);
        }
        $orderIncrementId=$shipmentExtensionAttributes->getOrderIncrementId();
        $carrierCode=$shipmentExtensionAttributes->getCarrierCode();
        if(!$carrierCode) $carrierCode="ups";
        $carrierTitle=$shipmentExtensionAttributes->getCarrierTitle();
        if(!$carrierTitle) $carrierTitle="UPS";
        $searchCriteria = $this->_searchCriteriaBuilder
            ->addFilter('increment_id', $orderIncrementId, 'eq')->create();
        $orderList = $this->_orderRepository->getList($searchCriteria);
        if(count($orderList)<>1){
            $message=__("Cannot find order %1", $orderIncrementId);
            $this->_logger->critical($message);
            throw new Magento\Sales\Exception\CouldNotShipException($message);
        }
        try{
            $order=$orderList->getFirstItem();   
            //In magento 2.2 empty item array is not handled any more. Build array with qty_to_ship.
            $items=array();
            foreach ($order->getAllItems() AS $orderItem) {
                if (! $orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                    continue;
                }
                $qtyShipped = $orderItem->getQtyToShip();
                $items[$orderItem->getId()] = $qtyShipped;
            }
            $tracks=$shipment_data->getTracks();
            $shipmentParameters = null;
            $number = "";
            if($tracks){
                $track=current($tracks);
                try{
                    $number = $track->getTrackNumber();
                    $shipmentParameters = array(
                        array(
                            "order_id" => $order->getId(),
                            "carrier_code" => $carrierCode,
                            "title" => $carrierTitle,
                            "number" => $number
                        )
                    );
                }catch(Exception $e){
                    
                }
            }
            $shipment=$this->_shipmentFactory->create(
                $order,
                $items,
                $shipmentParameters
            );
            $shipment->register();
            $shipment->getOrder()->setIsInProcess(true);
            $shipment->save();
            $order->addStatusHistoryComment(__("Order has been shipped: %1 ", $number), "shipped");
            $order->save();
            $notify=$this->_shipmentNotifier->notify($shipment);
            $shipment->save();
            $this->_response->setStatus("success");
            $this->_response->setMessage(__("Order %1 has been shipped:  %2", $order->getIncrementId(), $number));
            $this->_response->setFlow("update_shipment");            
            $this->_response->setOrderIncrementId($order->getIncrementId());            
            return $this->_response;
        }catch(Exception $e){
            $this->_logger->critical($e->getMessage());
            throw $e;
        }
    }
}

